@@ -2,8 +2,7 @@
 * Copyright (c) 2021-2023 Golubchikov Mihail <https://github.com/rue-ryuzaki>
 */
 
-#include <argparse/argparse.hpp>
-
+#include <argparse/argparse_decl.hpp>
 #include "./catch-define.h"
 
 TEST_CASE("1. argument nargs", "[argument]")
@@ -698,4 +697,4 @@ TEST_CASE("1. argument nargs", "[argument]")
         REQUIRE(parser.parse_args("a --bar c").to_string()
                                                          == "Namespace(args=['a', '--bar', 'c'], baz='123', bar=None)");
     }
-}
\ No newline at end of file
+}
